import { Sequelize, Optional, Model, DataTypes } from "sequelize";
import GenericExtr from "../../../data-storage/db/extr/generic-extr";

interface GenericExtrCreationAttributes extends Optional<GenericExtr, 'publicationCreationDate' | 'publicationUpdateDate'> { }

async function createGenericExtrModel(sequelize: Sequelize, tableName: string) {
  if (!tableName) {
    throw new Error("Table name is required");
  }

  // Obtiene la lista de tablas de la base de datos
  const tables = await sequelize.getQueryInterface().showAllTables();
    
  // Verifica si la tabla está en la lista
  if (!tables.includes(tableName)) {
    throw new Error(`The table "${tableName}" doesn't exists`)
  }

  return sequelize.define<Model<GenericExtr, GenericExtrCreationAttributes>>(tableName, {
    year: DataTypes.INTEGER,
    maker: DataTypes.STRING,
    model: DataTypes.STRING,
    version: DataTypes.STRING,
    color: DataTypes.STRING,
    price: DataTypes.INTEGER,
    title: DataTypes.STRING,
    description: DataTypes.TEXT,
    plate: DataTypes.STRING,
    mileage: DataTypes.INTEGER,
    gearbox: DataTypes.STRING,
    engine: DataTypes.STRING,
    fuel: DataTypes.STRING,
    imageUrl: {
      type: DataTypes.STRING,
      field: "image_url"
    },
    code: {
      type: DataTypes.STRING,
      primaryKey: true
    },
    url: DataTypes.STRING,
    region: DataTypes.STRING,
    city: DataTypes.STRING,
    sellerType: {
      type: DataTypes.STRING,
      field: "seller_type",
    },
    sellerName: {
      type: DataTypes.STRING,
      field: "seller_name",
    },
    publicationCreationDate: {
      type: DataTypes.DATE,
      allowNull: false,
      defaultValue: new Date(),
      field: "publication_creation"
    },
    publicationCreationRawDate: {
      type: DataTypes.DATE,
      allowNull: false,
      defaultValue: new Date(),
      field: "publication_creation_raw"
    },
    publicationUpdateDate: {
      type: DataTypes.DATE,
      allowNull: true,
      defaultValue: null,
      field: "publication_update"
    },
    publicationCreatedAt: {
      type: DataTypes.DATE,
      allowNull: false,
      defaultValue: new Date(),
      field: "createdAt"
    },
  }, {
    timestamps: false
  })
}

export default createGenericExtrModel;
