import * as dotenv from "dotenv";
import * as path from "path";
import * as log4js from "log4js";

export default class Log {
  private static _instance: Log;
  private _isTestingEnv: boolean;
  private _logger: log4js.Logger;

  private constructor() {
    this.initConfig();
    this.configureLog();
  }

  private initConfig() {
    this._isTestingEnv = process.env.NODE_ENV == "test";

    const basePath = path.join(__dirname, "../../");
    const confPath = this._isTestingEnv
      ? `${basePath}test/.env.test`
      : `${basePath}.env`;

    const result = dotenv.config({
      path: confPath
    });

    if (result.error) {
      throw new Error("Error retrieving config");
    }
  }

  private configureLog (): void {
    log4js.configure({
      appenders: {
        out: { type: "stdout", layout: { type: "coloured" } }
      },
      categories: {
        default: { appenders: ["out"], level: "debug" }
      }
    });

    this._logger = log4js.getLogger();
    this._logger.level = process.env.LOG_LEVEL || "info";
  }

  public static get instance() {
    return this._instance || (this._instance = new this());
  }

  public get logger() {
    return this._logger;
  }
}
